"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChanState = exports.SpecialChanType = exports.ChanType = void 0;
var ChanType;
(function (ChanType) {
    ChanType["CHANNEL"] = "channel";
    ChanType["LOBBY"] = "lobby";
    ChanType["QUERY"] = "query";
    ChanType["SPECIAL"] = "special";
})(ChanType || (exports.ChanType = ChanType = {}));
var SpecialChanType;
(function (SpecialChanType) {
    SpecialChanType["BANLIST"] = "list_bans";
    SpecialChanType["INVITELIST"] = "list_invites";
    SpecialChanType["CHANNELLIST"] = "list_channels";
    SpecialChanType["IGNORELIST"] = "list_ignored";
})(SpecialChanType || (exports.SpecialChanType = SpecialChanType = {}));
var ChanState;
(function (ChanState) {
    ChanState[ChanState["PARTED"] = 0] = "PARTED";
    ChanState[ChanState["JOINED"] = 1] = "JOINED";
})(ChanState || (exports.ChanState = ChanState = {}));
