import TextEncoder from './lib/TextEncoder'
import TextDecoder from './lib/TextDecoder'
import EncodingIndexes from './encoding-indexes'
import { getEncoding } from './lib'

//
// Implementation of Encoding specification
// https://encoding.spec.whatwg.org/
//

export { TextEncoder, TextDecoder, EncodingIndexes, getEncoding }