"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const msg_1 = __importDefault(require("../../models/msg"));
const msg_2 = require("../../../shared/types/msg");
exports.default = (function (irc, network) {
    const client = this;
    // If server supports CHGHOST cap, then changing the hostname does not require
    // sending PART and JOIN, which means less work for us over all
    irc.on("user updated", function (data) {
        network.channels.forEach((chan) => {
            const user = chan.findUser(data.nick);
            if (typeof user === "undefined") {
                return;
            }
            const msg = new msg_1.default({
                time: data.time,
                type: msg_2.MessageType.CHGHOST,
                new_ident: data.ident !== data.new_ident ? data.new_ident : "",
                new_host: data.hostname !== data.new_hostname ? data.new_hostname : "",
                self: data.nick === irc.user.nick,
                from: user,
            });
            chan.pushMessage(client, msg);
        });
    });
});
