"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const rootDir = process.env.NODE_ENV === "test" || process.env.NODE_ENV === "development"
    ? path_1.default.resolve(__dirname, "..")
    : path_1.default.resolve(__dirname, "..", "..");
function fromRoot(...fileName) {
    return path_1.default.resolve(rootDir, ...fileName);
}
exports.default = fromRoot;
