"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const msg_1 = __importDefault(require("../../models/msg"));
const msg_2 = require("../../../shared/types/msg");
exports.default = (function (irc, network) {
    const client = this;
    irc.on("info", function (data) {
        const lobby = network.getLobby();
        if (data.info) {
            const msg = new msg_1.default({
                type: msg_2.MessageType.MONOSPACE_BLOCK,
                command: "info",
                text: data.info,
            });
            lobby.pushMessage(client, msg, true);
        }
    });
});
