"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageType = void 0;
var MessageType;
(function (MessageType) {
    MessageType["UNHANDLED"] = "unhandled";
    MessageType["ACTION"] = "action";
    MessageType["AWAY"] = "away";
    MessageType["BACK"] = "back";
    MessageType["ERROR"] = "error";
    MessageType["INVITE"] = "invite";
    MessageType["JOIN"] = "join";
    MessageType["KICK"] = "kick";
    MessageType["LOGIN"] = "login";
    MessageType["LOGOUT"] = "logout";
    MessageType["MESSAGE"] = "message";
    MessageType["MODE"] = "mode";
    MessageType["MODE_CHANNEL"] = "mode_channel";
    MessageType["MODE_USER"] = "mode_user";
    MessageType["MONOSPACE_BLOCK"] = "monospace_block";
    MessageType["NICK"] = "nick";
    MessageType["NOTICE"] = "notice";
    MessageType["PART"] = "part";
    MessageType["QUIT"] = "quit";
    MessageType["CTCP"] = "ctcp";
    MessageType["CTCP_REQUEST"] = "ctcp_request";
    MessageType["CHGHOST"] = "chghost";
    MessageType["TOPIC"] = "topic";
    MessageType["TOPIC_SET_BY"] = "topic_set_by";
    MessageType["WHOIS"] = "whois";
    MessageType["RAW"] = "raw";
    MessageType["PLUGIN"] = "plugin";
    MessageType["WALLOPS"] = "wallops";
})(MessageType || (exports.MessageType = MessageType = {}));
