export declare const defaultMessages = "End-Of-Stream";
/**
 * Thrown on read operation of the end of file or stream has been reached
 */
export declare class EndOfStreamError extends Error {
    constructor();
}
