"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const msg_1 = __importDefault(require("../../models/msg"));
const msg_2 = require("../../../shared/types/msg");
exports.default = (function (irc, network) {
    const client = this;
    irc.on("part", function (data) {
        if (!data.channel) {
            return;
        }
        const chan = network.getChannel(data.channel);
        if (typeof chan === "undefined") {
            return;
        }
        const user = chan.getUser(data.nick);
        const msg = new msg_1.default({
            type: msg_2.MessageType.PART,
            time: data.time,
            text: data.message || "",
            hostmask: data.ident + "@" + data.hostname,
            from: user,
            self: data.nick === irc.user.nick,
        });
        chan.pushMessage(client, msg);
        if (data.nick === irc.user.nick) {
            client.part(network, chan);
        }
        else {
            chan.removeUser(user);
        }
    });
});
