"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const msg_1 = __importDefault(require("../../models/msg"));
const msg_2 = require("../../../shared/types/msg");
exports.default = (function (irc, network) {
    const client = this;
    irc.on("quit", function (data) {
        network.channels.forEach((chan) => {
            const user = chan.findUser(data.nick);
            if (typeof user === "undefined") {
                return;
            }
            const msg = new msg_1.default({
                time: data.time,
                type: msg_2.MessageType.QUIT,
                text: data.message || "",
                hostmask: data.ident + "@" + data.hostname,
                from: user,
            });
            chan.pushMessage(client, msg);
            chan.removeUser(user);
        });
        // If user with the nick we are trying to keep has quit, try to get this nick
        if (network.keepNick === data.nick) {
            irc.changeNick(network.keepNick);
            network.keepNick = null;
        }
    });
});
